@@ -27,13 +27,24 @@ from array import array
 import types
 
 # Python 2/3 compatibility.
-from six import binary_type, byte2int, indexbytes, iterbytes
+from six import PY3, binary_type, byte2int, indexbytes, iterbytes
 
 # Xlib modules
 from .. import X
 from ..support import lock
 
 
+def decode_string(bs):
+    return bs.decode('ascii')
+
+if PY3:
+    def encode_array(a):
+        return a.tobytes()
+else:
+    def encode_array(a):
+        return a.tostring()
+
+
 class BadDataError(Exception): pass
 
 # These are struct codes, we know their byte sizes
@@ -424,17 +435,14 @@ class String8(ValueField):
 
     def parse_binary_value(self, data, display, length, format):
         if length is None:
-            return data.decode(), b''
+            return decode_string(data), b''
 
         if self.pad:
             slen = length + ((4 - length % 4) % 4)
         else:
             slen = length
 
-        if sys.version_info < (3, 0):
-            data_str = data[:length]
-        else:
-            data_str = data[:length].decode()
+        data_str = decode_string(data[:length])
 
         return data_str, data[slen:]
 
@@ -552,8 +560,8 @@ class List(ValueField):
         if self.type.structcode and len(self.type.structcode) == 1:
             if self.type.check_value is not None:
                 val = [self.type.check_value(v) for v in val]
-            data = array(struct_to_array_codes[self.type.structcode],
-                               val).tostring()
+            a = array(struct_to_array_codes[self.type.structcode], val)
+            data = encode_array(a)
         else:
             data = []
             for v in val:
@@ -685,7 +693,8 @@ class PropertyData(ValueField):
                 val = list(val)
 
             size = fmt // 8
-            data = array(array_unsigned_codes[size], val).tostring()
+            a = array(array_unsigned_codes[size], val)
+            data = encode_array(a)
             dlen = len(val)
 
         dl = len(data)
@@ -806,7 +815,7 @@ class KeyboardMapping(ValueField):
             for i in range(len(v), keycodes):
                 a.append(X.NoSymbol)
 
-        return a.tostring(), len(value), keycodes
+        return encode_array(a), len(value), keycodes
 
 
 class ModifierMapping(ValueField):
@@ -837,7 +846,7 @@ class ModifierMapping(ValueField):
             for i in range(len(v), keycodes):
                 a.append(0)
 
-        return a.tostring(), len(value), keycodes
+        return encode_array(a), len(value), keycodes
 
 class EventField(ValueField):
     structcode = None
@@ -903,7 +912,7 @@ class StrClass(object):
 
     def parse_binary(self, data, display):
         slen = byte2int(data) + 1
-        return data[1:slen].decode(), data[slen:]
+        return decode_string(data[1:slen]), data[slen:]
 
 Str = StrClass()
 
